<?php

require_once("3rdparty/domain/IRegistrar.php");
require_once("3rdparty/domain/standardfunctions.php");
require_once("3rdparty/domain/xynta/master.php");

class Xynta implements IRegistrar
{
    public $User;
    public $Password;
    
    public $Error;
    public $Warning;
    public $Success;

    public $Period = 1;
    public $registrarHandles = array();
    
    private $Xynta;
    
	function __construct(){

		$this->Xynta = __CLASS__;
		
		$this->Error = array();
		$this->Warning = array();
		$this->Success = array();
        
	}

	function checkDomain($domain){

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'check',
            'Domain'        => $domain
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['results'][0]['status'] == 'available'){
            return true;
        } elseif($curlResponse[0]['results'][0]['status'] == 'unavailable'){
            return false;
        } else {
            $this->Error[] = 'checkDomain returned unknown status';
            return false;
        }

	}

	function registerDomain($domain, $nameservers = array(), $whois = null, $authcode = ""){

        $cOwnerHandle = $this->getContactHandle($whois,HANDLE_OWNER);

        if($cOwnerHandle==""){
            $this->createContact($whois, HANDLE_OWNER);
            $cOwnerHandle = $this->getContactHandle($whois,HANDLE_OWNER);
        }

        $cAdminHandle = $this->getContactHandle($whois,HANDLE_ADMIN);

        if($cAdminHandle==""){
            $this->createContact($whois, HANDLE_ADMIN);
            $cAdminHandle = $this->getContactHandle($whois,HANDLE_ADMIN);
        }

        $cTechHandle = $this->getContactHandle($whois,HANDLE_TECH);

        if($cTechHandle==""){
            $this->createContact($whois, HANDLE_TECH);
            $cTechHandle = $this->getContactHandle($whois,HANDLE_TECH);
        }

        $getDNS1 = $nameservers["ns1"];
        $getDNS2 = $nameservers["ns2"];
        $getDNS3 = $nameservers["ns3"];

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'order',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2),
            'OwnerHandle'   => $cOwnerHandle,
            'AdminHandle'   => $cAdminHandle,
            'TechHandle'    => $cTechHandle,
            'DNS1'          => $getDNS1,
            'DNS2'          => $getDNS2,
            'DNS3'          => $getDNS3,
            'AuthKey'       => $authcode
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return true;

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] = sprintf("Xynta: Error while registering '%s': %s", $domain, $error_msg);
            return false;

        }

	}

	function transferDomain($domain, $nameservers = array(), $whois = null, $authcode = ""){

        return $this->registerDomain($domain, $nameservers, $whois, $authcode);

	}

	function deleteDomain($domain, $delType = 'end'){

        $sType = NULL;

        if($delType == "end") {
            $sType = "cancel";
        } elseif($delType == "now") {
            $sType = "cancel-direct";
        }

        $setPostValues = [
            'uid' => $this->User,
            'hash' => $this->Password,
            'm' => 'domains',
            'a' => 'del',
            't' => $sType,
            'Domain' => parseDomain($domain, 1),
            'Tld' => parseDomain($domain, 2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return true;

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] = sprintf("Xynta: Error while terminating '%s': %s", $domain, $error_msg);
            return false;

        }

	}

	function getDomainInformation($domain){

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'show',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            $nameservers_array = array($curlResponse[0]['results'][0]['dns1'],$curlResponse[0]['results'][0]['dns2'],$curlResponse[0]['results'][0]['dns3']);

            $whois = new whois();
            $whois->ownerHandle = $curlResponse[0]['results'][0]['owner_handle']['id'];
            $whois->adminHandle = $curlResponse[0]['results'][0]['admin_handle']['id'];
            $whois->techHandle 	= $curlResponse[0]['results'][0]['tech_handle']['id'];

            $expirationdate = $curlResponse[0]['results'][0]['expiration_date'];
            $registrationdate = $curlResponse[0]['results'][0]['registration_date'];
            $registrationdate = explode(" ",$registrationdate);
            $registrationdate = $registrationdate[0];
            $authkey = $curlResponse[0]['results'][0]['auth_key'];
            $status = $curlResponse[0]['results'][0]['status'];

            $response = array(
                "Domain" => $domain,
                "Information" => array(
                    "nameservers" => $nameservers_array,
                    "whois" => $whois,
                    "expiration_date" => $expirationdate,
                    "registration_date" => $registrationdate,
                    "authkey" => $authkey
                )
            );
            return $response;

        } else {

            $error_msg = $curlResponse[0]['status'];

            $this->Error[] 	= sprintf("Xynta: Error while retrieving information about '%s': %s", $domain, $error_msg);
            return false;

        }

	}

	public function getDomainList($contactHandle = ""){

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'list'
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            $domain_array = array();

            foreach($curlResponse[0]['results'] as $i => $value) {

                $sld = $value['domain'];
                $tld = $value['tld'];
                $domain = "$sld.$tld";

                $nameservers_array = array($value['dns1'],$value['dns2'],$value['dns3']);
                $whois = new whois();
                $whois->ownerHandle = $value['owner_handle'];
                $whois->adminHandle = $value['admin_handle'];
                $whois->techHandle = $value['tech_handle'];
                $registration_date = explode(" ",$value['registration_date']);

                $expirationdate = $value['expiration_date'];
                $registrationdate = $registration_date[0];
                $authkey = $value['auth_key'];

                // Return array with data
                $response = array(
                    "Domain" => $domain,
                    "Information" => array(
                        "nameservers" => $nameservers_array,
                        "whois" => $whois,
                        "expiration_date" => $expirationdate,
                        "registration_date" => $registrationdate,
                        "authkey" => $authkey
                    )
                );
                $domain_array[] = $response;

            }

            return $domain_array;

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] 	= sprintf("Xynta: Error while retrieving domains: %s", $error_msg);
            return false;

        }

	}

	function lockDomain($domain, $lock = true){

        if($lock == true){
            $sAction = "lock";
        } else {
            $sAction = "unlock";
        }

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => $sAction,
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return true;

        } else {

            if($curlResponse[0]['status'] == "not-supported" || $curlResponse[0]['status'] == "not-authorised" || $curlResponse[0]['status'] == "not-found") {
                $error_msg = $curlResponse[0]['status'];
            } else {
                $error_msg = $curlResponse[0]['response'];
            }
            $this->Error[] = sprintf("Xynta: Error while (un)locking: $error_msg");

        }

	}

	function setDomainAutoRenew($domain, $autorenew = true){

        $sType = NULL;

        if($autorenew == true) {
            $sType = "undo";
        } else {
            $sType = "cancel";
        }

        $setPostValues = [
            'uid' => $this->User,
            'hash' => $this->Password,
            'm' => 'domains',
            'a' => 'del',
            't' => $sType,
            'Domain' => parseDomain($domain, 1),
            'Tld' => parseDomain($domain, 2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return true;

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] = sprintf("Xynta: Error while changing autorenew setting '%s': %s", $domain, $error_msg);
            return false;

        }

	}

    public function getToken($domain){

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'get-authkey',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return $curlResponse[0]['results'][0]['auth_key'];

        } else {

            $error_msg = $curlResponse[0]['results'][0]['response'];
            $this->Error[] 	= sprintf("Xynta: Error while retrieving auth key: %s", $error_msg);

        }

    }

    public function getSyncData($list_domains){

        $limit   = 10;
        $checked = 0;

        foreach ($list_domains as $domain => &$value) {

            $data = $this->getDomainInformation($domain);

            if ($data === false) {
                $value['Status']    = 'error';
                $value['Error_msg'] = 'Domain name not found or an Internal Server Error occurred';

                continue;
            }

            $checked++;

            if ($limit !== null && $checked > $limit) {
                break;
            }

            $value = $data;

            $value['Information']['nameservers']        = $data['Information']['nameservers'];
            $value['Information']['expiration_date']    = isset($data['Information']['expiration_date']) ? $data['Information']['expiration_date'] : '';
            $value['Information']['auto_renew']         = empty($data['Information']['auto_renew']) ? '' : $data['Information']['auto_renew'];

            $value['Status'] = 'success';
        }

        unset($value);

        return $list_domains;

	}

	function updateDomainWhois($domain,$whois){

        $cOwnerHandle = $this->getContactHandle($whois,HANDLE_OWNER);
        $cAdminHandle = $this->getContactHandle($whois,HANDLE_ADMIN);
        $cTechHandle = $this->getContactHandle($whois,HANDLE_TECH);

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'edit-contact',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2),
            'OwnerHandle'   => $cOwnerHandle,
            'AdminHandle'   => $cAdminHandle,
            'TechHandle'    => $cTechHandle
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {
            return true;
        } else {
            $error_msg = $curlResponse[0]['status'];
            $this->Error[] = sprintf("Xynta: Error while changing contact for '%s': %s", $domain, $error_msg);
            return false;
        }

	}

	function getDomainWhois($domain){

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'show',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2)
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            $whois = array();
            $whois['ownerHandle'] = $curlResponse[0]['results'][0]['owner_handle']['id'];
            $whois['adminHandle'] = $curlResponse[0]['results'][0]['admin_handle']['id'];
            $whois['techHandle'] = $curlResponse[0]['results'][0]['tech_handle']['id'];

            return $whois;

        } else {

            $error_msg = $curlResponse[0]['status'];

            $this->Error[] 	= sprintf("Xynta: Error while retrieving information about '%s': %s", $domain, $error_msg);
            return false;

        }

	}

	function createContact($whois, $type = HANDLE_OWNER){

        $cContact = getWHOISVariables($whois,$type);
        $cUnixTimestamp = date("U");

        $cHandle = NULL;
        $cInitials = NULL;
        $cSurName = NULL;
        $cCompanyName = NULL;

        $cInitials = $cContact['Initials'];
        $cSurName = $cContact['SurName'];
        $cCompanyName = $cContact['CompanyName'];

        if($cContact['Handle'] != ""){
            $cHandle = $cContact['Handle'];
        } elseif($cContact['CompanyName'] != ""){
            $cHandle = "$cCompanyName-$cUnixTimestamp";
        } else {
            $cHandle = "$cSurName$cInitials-$cUnixTimestamp";
        }

        $cHandle = prepareHandle($cHandle);

        $setPostValues = [
            'uid' => $this->User,
            'hash' => $this->Password,
            'm' => 'domains',
            'a' => 'add',
            't' => 'contact',
            'Handle' => $cHandle,
            'CompanyName' => $cContact['CompanyName'],
            'CompanyNumber' => $cContact['CompanyNumber'],
            'TaxNumber' => $cContact['TaxNumber'],
            'Initials' => $cContact['Initials'],
            'SurName' => $cContact['SurName'],
            'Address' => $cContact['Address'],
            'ZipCode' => $cContact['ZipCode'],
            'City' => $cContact['City'],
            'Country' => $cContact['Country'],
            'EmailAddress' => $cContact['EmailAddress'],
            'PhoneNumber' => $cContact['PhoneNumber']
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return $curlResponse[0]['results'][0]['handle'];

        } else {

            $error_msg = $curlResponse[0]['status'];

            $this->Error[] 	= sprintf("Xynta: Error while creating contact: $error_msg");
            return false;

        }

	}

	function updateContact($handle, $whois, $type = HANDLE_OWNER){

        $cContact = getWHOISVariables($whois,$type);

        $setPostValues = [
            'uid' => $this->User,
            'hash' => $this->Password,
            'm' => 'domains',
            'a' => 'edit',
            't' => 'contact',
            'Handle' => $handle,
            'CompanyName' => $cContact['CompanyName'],
            'CompanyNumber' => $cContact['CompanyNumber'],
            'TaxNumber' => $cContact['TaxNumber'],
            'Initials' => $cContact['Initials'],
            'SurName' => $cContact['SurName'],
            'Address' => $cContact['Address'],
            'ZipCode' => $cContact['ZipCode'],
            'City' => $cContact['City'],
            'Country' => $cContact['Country'],
            'EmailAddress' => $cContact['EmailAddress'],
            'PhoneNumber' => $cContact['PhoneNumber']
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return $curlResponse[0]['results'][0]['handle'];

        } else {

            $error_msg = $curlResponse[0]['status'];

            $this->Error[] 	= sprintf("Xynta: Error while updating contact: $error_msg");
            return false;

        }

	}

    function getContact($handle){

        $whois = new whois();

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'show',
            't'             => 'contact',
            'Handle'        => $handle
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            if($curlResponse[0]['results'][0]['company_name'] == "null" || $curlResponse[0]['results'][0]['company_name'] == ""){
                $whois->ownerCompanyName = null;
            } else {
                $whois->ownerCompanyName = $curlResponse[0]['results'][0]['company_name'];
            }
            if($curlResponse[0]['results'][0]['company_number'] == "null" || $curlResponse[0]['results'][0]['company_number'] == ""){
                $whois->ownerCompanyNumber = null;
            } else {
                $whois->ownerCompanyNumber = $curlResponse[0]['results'][0]['company_number'];
            }
            if($curlResponse[0]['results'][0]['tax_number'] == "null" || $curlResponse[0]['results'][0]['tax_number'] == ""){
                $whois->ownerTaxNumber = null;
            } else {
                $whois->ownerTaxNumber = $curlResponse[0]['results'][0]['tax_number'];
            }
            $whois->ownerCompanyLegalForm = null;

            $whois->ownerSex      = null;
            $whois->ownerInitials = $curlResponse[0]['results'][0]['initials'];
            $whois->ownerSurName  = $curlResponse[0]['results'][0]['surname'];
            $whois->ownerAddress  = $curlResponse[0]['results'][0]['address'];
            $whois->ownerZipCode  = $curlResponse[0]['results'][0]['zipcode'];
            $whois->ownerCity     = $curlResponse[0]['results'][0]['city'];

            $whois->ownerCountry      = $curlResponse[0]['results'][0]['country'];
            $whois->ownerEmailAddress = $curlResponse[0]['results'][0]['email_address'];
            $whois->ownerPhoneNumber  = $curlResponse[0]['results'][0]['phone_number'];
            $whois->ownerFaxNumber    = null;

            return $whois;

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] 	= sprintf("Xynta: Error while retrieving contact: $error_msg");
            return $whois;

        }

	}

    function getContactHandle($whois = array(), $type = HANDLE_OWNER){

        $cContact = getWHOISVariables($whois,$type);

        $setPostValues = [
            'uid' => $this->User,
            'hash' => $this->Password,
            'm' => 'domains',
            'a' => 'check',
            't' => 'contact',
            'Initials' => $cContact['Initials'],
            'SurName' => $cContact['SurName'],
            'CompanyName' => $cContact['CompanyName']
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return $curlResponse[0]['results'][0]['handle'];

        } else {

            return false;

        }

   	}

    function getContactList($surname = ""){

        $contact_list = array();

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'list',
            't'             => 'contact'
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            foreach ($curlResponse[0]['results'] as $i => $value) {

                $contact = array();
                $contact['Handle'] = $value['handle'];
                if($value['company_name'] != "" && $value['company_name'] != "null") {
                    $contact['CompanyName'] = $value['company_name'];
                }
                $contact['SurName'] = $value['surname'];
                $contact['Initials'] = $value['initials'];
                $contact_list[] = $contact;

            }

            if (count($contact_list) > 0) {
                return $contact_list;
            } else {
                $this->Error[] 	= sprintf("Xynta: No importable contacts found");
                return false;
            }

        } else {

            $error_msg = $curlResponse[0]['status'];
            $this->Error[] 	= sprintf("Xynta: Error while retrieving contacts: $error_msg");
            return false;

        }

   	}

   	function updateNameServers($domain, $nameservers = array()){

        $getDNS1 = $nameservers["ns1"];
        $getDNS2 = $nameservers["ns2"];
        $getDNS3 = $nameservers["ns3"];

        $setPostValues = [
            'uid'           => $this->User,
            'hash'          => $this->Password,
            'm'             => 'domains',
            'a'             => 'edit-nameserver',
            'Domain'        => parseDomain($domain,1),
            'Tld'           => parseDomain($domain,2),
            'DNS1'          => $getDNS1,
            'DNS2'          => $getDNS2,
            'DNS3'          => $getDNS3
        ];

        $curlResponse = APICall($setPostValues);

        if($curlResponse[0]['status'] == "success") {

            return true;

        } else {

            $error_msg = $curlResponse[0]['results'][0]['response'];
            $this->Error[] 	= sprintf("Xynta: Error while updating nameservers: %s", $error_msg);

        }

	}

    function getDNSTemplates(){

    }

    function getDNSZone($domain){

    }

    function saveDNSZone($domain, $dns_zone){

    }

	static function getVersionInformation(){
		require_once("3rdparty/domain/xynta/version.php");
		return $version;	
	}	
	
}
